/* Filename:  search.cpp  (for QSearch 2.1 software)

   Author:  Br. David Carlson

   Uses Win2K getmeta script written by Brian LaSitis.

   Date:  February 23, 2001

   Revised:  May 25, 2001 to check that the strings in Key1, Key2, Key3 have
   room in them to hold a beginning and ending # character.

   Revised:  June 25, 2001 to process both the Linux keywordfile and a similar
   keywordfile.txt transferred from our Win2K server.

   Minor updates:  December 22, 2002; January 25, 2003; May 20, 2003;
      June 27, 2003

   This CGI program looks for lines that contain keywords given by WWW_Key1,
   WWW_Key2, WWW_Key3 (as many as are nonempty) in the keywordfiles
   created by the getmeta scripts.  These files must be in the same directory
   as the compiled version of this program.  The user supplies input by
   using the search.html file via a web browser.  The output includes each
   matching URL and the description field, all in HTML format.  This output
   will, or course, be sent to the user's browser by the web server.
 
   This program could be modified so that the count of the number of
   uses of the search engine is displayed to the web browser user, 
   or the count could be eliminated totally.  At present, the count
   is kept in the search.count file, found in the same directory, but
   is not displayed.
  
   To compile this program, use the following at the Linux command line:
   g++ search.cpp stringhelp.cpp -o search -s
 
   This software is provided "as is" without warranty of any kind.  The user
   assumes all of the risk in using the software.  Although the author retains
   the copyright to the software, blanket permission is given to freely use,
   copy, and modify the software as long as the files still contain the author's
   copyright notice; and the software, or any modification of it, remains as
   freeware or shareware.  Should you wish to use this code as part of a
   commercial product, contact the author, carlsond@stvincent.edu, for 
   permission.
 
   Copyright 2001-2003, Br. David Carlson, St. Vincent Archabbey, Latrobe, PA
*/

#include "stringhelp.h"

// Change the following line to the top URL on Linux, the URL under which all
// of your web files are located:
#define TOPURL     "http://cis.stvincent.edu"

// Change the following to give the top URL on the Win2K server:
#define TOPURL2    "http://cis2.stvincent.edu"

// Next line should give name of file containing opening HTML for search results:
#define HEAD       "head.html"

// Next line should give name of data file created by Linux getmeta script:
#define KEYFILE    "keywordfile"

// Next line should give name of data file created by Win2K getmeta script:
#define KEYFILE2   "keywordfile.txt"

// Next line should give name of text file containing usage count for search:
#define COUNTFILE  "search.count"


void SearchFile(fstream & fs, fstream & fs2);
void Print(char * Filename);
void IncrementCount(char * Filename);


int main(void)
   {
   fstream InFile, InFile2;

   InFile.open(KEYFILE, ios::in);
   if (InFile.fail())
      exit(1);

   InFile2.open(KEYFILE2, ios::in);
   if (InFile2.fail())
      exit(1);

   IncrementCount(COUNTFILE);
   SearchFile(InFile, InFile2);
   InFile.close();
   InFile2.close();

   return 0;
   }


/* Given:   fs   Text file stream already opened for input.
            fs2  A second text file stream opened for input.
   Task:    To carry out the search of these files for the keyword(s) given
            in the 3 web variables: WWW_Key1, WWW_Key2, and WWW_Key3.
            HTML-formatted output is produced (which the web server will
            send to the user's browser).  For each matching line this
            output contains the URL and the description.
   Return:  Nothing.
*/
void SearchFile(fstream & fs, fstream & fs2)
   {
   StringType Key1, Key2, Key3, Line, URL, Description;
   StringType Target1 = "#";
   StringType Target2 = "#";
   StringType Target3 = "#";
   int CharCount, MatchCount, KeyCount, KeyLen1, KeyLen2, KeyLen3;
   const char Quote = 34;
   bool Match;

   GetValue("WWW_Key1", Key1);
   GetValue("WWW_Key2", Key2);
   GetValue("WWW_Key3", Key3);
   KeyLen1 = strlen(Key1);
   KeyLen2 = strlen(Key2);
   KeyLen3 = strlen(Key3);

   cout << "Content-type: text/html" << endl << endl;
   Print(HEAD);
   cout << "<OL>" << endl;

   if ((KeyLen1 == 0) && (KeyLen2 == 0) && (KeyLen3 == 0))
      {
      cout <<
         "<LI>No keyword was supplied or keyword contained disallowed characters"
         << endl;
      cout << "</OL></BODY></HTML>" << endl;
      return;
      }

   // Added May 25, 2001:
   int StrMax3 = StrMax - 3;
   if ((KeyLen1 > StrMax3) || (KeyLen2 > StrMax3) || (KeyLen3 > StrMax3))
      {
      cout << "<LI>Keyword was too long" << endl;
      cout << "</OL></BODY></HTML>" << endl;
      return;
      }
   // End May 25, 2001 addition.

   strcat(Target1, Key1);
   strcat(Target1, "#");
   strcat(Target2, Key2);
   strcat(Target2, "#");
   strcat(Target3, Key3);
   strcat(Target3, "#");

   CharCount = MyGetLine(fs, Line, StrMax);
   MatchCount = 0;

   while (CharCount > 0)
      {
      if (KeyLen1 > 0)
         Match = SubstringPresent(Line, Target1);
      else
         Match = true;
      if (Match && (KeyLen2 > 0))
         Match = SubstringPresent(Line, Target2);
      if (Match && (KeyLen3 > 0))
         Match = SubstringPresent(Line, Target3);

      if (Match)
         if (GetSubstring(Line, '/', '|', URL))
            {
            MatchCount++;
            cout << "<LI><A HREF="
               << Quote
               << TOPURL << '/'
               << URL << Quote << ">" << TOPURL << '/'
               << URL << "</A>" << endl;
            if (GetSubstring(Line, '|', '#', Description))
               cout << Description << endl;
            }
      CharCount = MyGetLine(fs, Line, StrMax);
      }

   //*** Section added June 25, 2001 to process keywordfile from Win2K server:
   CharCount = MyGetLine(fs2, Line, StrMax);

   while (CharCount > 0)
      {
      if (KeyLen1 > 0)
         Match = SubstringPresent(Line, Target1);
      else
         Match = true;
      if (Match && (KeyLen2 > 0))
         Match = SubstringPresent(Line, Target2);
      if (Match && (KeyLen3 > 0))
         Match = SubstringPresent(Line, Target3);

      if (Match)
         if (GetSubstring(Line, '/', '|', URL))
            {
            MatchCount++;
            cout << "<LI><A HREF="
               << Quote
               << TOPURL2 << '/'
               << URL << Quote << ">" << TOPURL2 << '/'
               << URL << "</A>" << endl;
            if (GetSubstring(Line, '|', '#', Description))
               cout << Description << endl;
            }
      CharCount = MyGetLine(fs2, Line, StrMax);
      }
   //*** End of section added June 25, 2001

   if (MatchCount == 0)
      cout << "<LI>No matches were found" << endl;

   cout << "</OL><P><SMALL>Powered by QSearch 2.1 freeware written by" << endl; 
   cout << "<A HREF=" << Quote << "http://cis.stvincent.edu/carlsond" << endl;
   cout << Quote << ">Br. David Carlson</A></SMALL></P></BODY></HTML>" << endl;
   }


/* Given:   Filename    String containing name of a text file to output.
   Task:    To copy the contents of this file to standard output.
   Return:  Nothing.
*/
void Print(char * Filename)
   {
   fstream fs;
   StringType Line;

   fs.open(Filename, ios::in);
   if (fs.fail())
      exit(1);

   MyGetLine(fs, Line, StrMax);

   while (! fs.fail())
      {
      cout << Line << endl;
      MyGetLine(fs, Line, StrMax);
      }

   fs.close();
   }


/* Given:   Filename   String containing name of a text file.
   Task:    To increment the integer count contained in this file.
   Return:  Nothing.
*/
void IncrementCount(char * Filename)
   {
   fstream InFile, OutFile;
   int Count;

   InFile.open(Filename, ios::in);
   if (InFile.fail())
      return;   // just ignore the count if the file cannot be opened

   InFile >> Count;
   Count++;
   InFile.close();

   OutFile.open(Filename, ios::out);
   if (OutFile.fail())
      return;   // just ignore the count if the file cannot be opened

   OutFile << Count;
   OutFile.close();
   }

